@@ -91,6 +91,7 @@ class OpenEVSE:
         self.websocket: OpenEVSEWebsocket | None = None
         self.callback: Callable | None = None
         self._loop = None
+        self.tasks = None
 
     async def process_request(
         self,
@@ -241,8 +242,8 @@ class OpenEVSE:
 
         if not self._ws_listening:
             _LOGGER.debug("Setting up websocket ping...")
-            self._loop.create_task(self.repeat(300, self.websocket.keepalive))
             self._loop.create_task(self.websocket.listen())
+            self._loop.create_task(self.repeat(300, self.websocket.keepalive))
             pending = asyncio.all_tasks()
             self._ws_listening = True
             try:
@@ -314,7 +315,7 @@ class OpenEVSE:
 
         *args and **kwargs are passed as the arguments to func.
         """
-        while True:
+        while self.ws_state != "stopped":
             await asyncio.sleep(interval)
             await func(*args, **kwargs)
 
